from __future__ import annotations

from prettyqt.qt import QtDesigner


class PyDesignerCustomWidgetCollection(QtDesigner.QPyDesignerCustomWidgetCollection):
    pass


if __name__ == "__main__":
    fact = PyDesignerCustomWidgetCollection()
